package RIFF::Info::ASF;

use strict;
use warnings;
use vars qw($VERSION);
$VERSION = '1.00';

1;

__END__
# Below is stub documentation for your module. You better edit it!

=head1 NAME

RIFF::Info::ASF - See RIFF::Info for your ASF attribute needs.
This module is just here to help you find ASF by keyword search on
CPAN.

=head1 AUTHOR

Allen Day <allenday@ucla.edu>
Copyright (c) 2002, Allen Day

You may enjoy this module under the same terms as Perl itself.

=cut
